@@ -1,7 +1,7 @@
 /*
  * irc-channel.c - channel and private chat management for IRC plugin
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -33,12 +33,13 @@
 #include "irc-color.h"
 #include "irc-command.h"
 #include "irc-config.h"
+#include "irc-input.h"
 #include "irc-join.h"
 #include "irc-modelist.h"
 #include "irc-nick.h"
 #include "irc-protocol.h"
 #include "irc-server.h"
-#include "irc-input.h"
+#include "irc-upgrade.h"
 
 
 char *irc_channel_typing_state_string[IRC_CHANNEL_NUM_TYPING_STATES] =
@@ -48,6 +49,10 @@ char *irc_channel_typing_state_string[IRC_CHANNEL_NUM_TYPING_STATES] =
 char *irc_channel_default_chantypes = "#&";
 
 
+char *irc_channel_get_buffer_input_prompt (struct t_irc_server *server,
+                                           struct t_irc_channel *channel);
+
+
 /*
  * Checks if a channel pointer is valid for a server.
  *
@@ -75,90 +80,6 @@ irc_channel_valid (struct t_irc_server *server, struct t_irc_channel *channel)
     return 0;
 }
 
-/*
- * Moves new channel/pv buffer near server.
- */
-
-void
-irc_channel_move_near_server (struct t_irc_server *server, int channel_type,
-                              struct t_gui_buffer *buffer)
-{
-    int number, number_channel, number_last_channel, number_last_private;
-    int number_found;
-    char str_number[32];
-    const char *ptr_type, *ptr_server_name;
-    struct t_hdata *hdata_buffer;
-    struct t_gui_buffer *ptr_buffer;
-
-    number = weechat_buffer_get_integer (buffer, "number");
-    number_last_channel = 0;
-    number_last_private = 0;
-    number_found = 0;
-
-    hdata_buffer = weechat_hdata_get ("buffer");
-    ptr_buffer = weechat_hdata_get_list (hdata_buffer, "gui_buffers");
-    while (ptr_buffer)
-    {
-        if ((ptr_buffer != buffer)
-            && (weechat_buffer_get_pointer (ptr_buffer,
-                                            "plugin") == weechat_irc_plugin))
-        {
-            ptr_type = weechat_buffer_get_string (ptr_buffer,
-                                                  "localvar_type");
-            ptr_server_name = weechat_buffer_get_string (ptr_buffer,
-                                                         "localvar_server");
-            number_channel = weechat_buffer_get_integer (ptr_buffer,
-                                                         "number");
-            if (ptr_type && ptr_type[0]
-                && ptr_server_name && ptr_server_name[0]
-                && (strcmp (ptr_server_name, server->name) == 0))
-            {
-                if (strcmp (ptr_type, "channel") == 0)
-                {
-                    if (number_channel > number_last_channel)
-                        number_last_channel = number_channel;
-                }
-                else if (strcmp (ptr_type, "private") == 0)
-                {
-                    if (number_channel > number_last_private)
-                        number_last_private = number_channel;
-                }
-            }
-        }
-        /* move to next buffer */
-        ptr_buffer = weechat_hdata_move (hdata_buffer, ptr_buffer, 1);
-    }
-
-    /* use last channel/pv number + 1 */
-    switch (channel_type)
-    {
-        case IRC_CHANNEL_TYPE_CHANNEL:
-            if (number_last_channel > 0)
-                number_found = number_last_channel + 1;
-            break;
-        case IRC_CHANNEL_TYPE_PRIVATE:
-            if (number_last_private > 0)
-                number_found = number_last_private + 1;
-            else if (number_last_channel > 0)
-                number_found = number_last_channel + 1;
-            break;
-    }
-
-    if ((number_found == 0)
-        && (weechat_config_integer (irc_config_look_server_buffer) ==
-            IRC_CONFIG_LOOK_SERVER_BUFFER_INDEPENDENT))
-    {
-        number_found = weechat_buffer_get_integer (server->buffer, "number") + 1;
-    }
-
-    /* switch to number found */
-    if ((number_found >= 1) && (number_found != number))
-    {
-        snprintf (str_number, sizeof (str_number), "%d", number_found);
-        weechat_buffer_set (buffer, "number", str_number);
-    }
-}
-
 /*
  * Searches for a channel by name.
  *
@@ -268,7 +189,7 @@ irc_channel_create_buffer (struct t_irc_server *server,
     struct t_hashtable *buffer_props;
     int buffer_created, current_buffer_number, buffer_position;
     int autojoin_join, manual_join, noswitch;
-    char str_number[32], *channel_name_lower, *buffer_name;
+    char str_number[32], *channel_name_lower, *buffer_name, *prompt;
     const char *short_name, *localvar_channel;
 
     buffer_created = 0;
@@ -308,6 +229,9 @@ irc_channel_create_buffer (struct t_irc_server *server,
             buffer_props,
             "localvar_set_type",
             (channel_type == IRC_CHANNEL_TYPE_CHANNEL) ? "channel" : "private");
+        prompt = irc_channel_get_buffer_input_prompt (server, NULL);
+        weechat_hashtable_set (buffer_props, "input_prompt", (prompt) ? prompt : "");
+        free (prompt);
         weechat_hashtable_set (buffer_props, "localvar_set_nick", server->nick);
         weechat_hashtable_set (buffer_props, "localvar_set_host", server->host);
         weechat_hashtable_set (buffer_props, "localvar_set_server", server->name);
@@ -325,7 +249,8 @@ irc_channel_create_buffer (struct t_irc_server *server,
 
     if (ptr_buffer)
     {
-        weechat_nicklist_remove_all (ptr_buffer);
+        if (!irc_upgrading)
+            weechat_nicklist_remove_all (ptr_buffer);
         weechat_hashtable_map (buffer_props, &irc_channel_apply_props, ptr_buffer);
     }
     else
@@ -333,7 +258,7 @@ irc_channel_create_buffer (struct t_irc_server *server,
         ptr_buffer_for_merge = NULL;
         if (channel_type == IRC_CHANNEL_TYPE_PRIVATE)
         {
-            switch (weechat_config_integer (irc_config_look_pv_buffer))
+            switch (weechat_config_enum (irc_config_look_pv_buffer))
             {
                 case IRC_CONFIG_LOOK_PV_BUFFER_MERGE_BY_SERVER:
                     /* merge private buffers by server */
@@ -359,8 +284,8 @@ irc_channel_create_buffer (struct t_irc_server *server,
         if (weechat_buffer_get_integer (ptr_buffer, "layout_number") < 1)
         {
             buffer_position = (channel_type == IRC_CHANNEL_TYPE_CHANNEL) ?
-                weechat_config_integer (irc_config_look_new_channel_position) :
-                weechat_config_integer (irc_config_look_new_pv_position);
+                weechat_config_enum (irc_config_look_new_channel_position) :
+                weechat_config_enum (irc_config_look_new_pv_position);
             switch (buffer_position)
             {
                 case IRC_CONFIG_LOOK_BUFFER_POSITION_NONE:
@@ -374,8 +299,11 @@ irc_channel_create_buffer (struct t_irc_server *server,
                     break;
                 case IRC_CONFIG_LOOK_BUFFER_POSITION_NEAR_SERVER:
                     /* move buffer after last channel/pv of server */
-                    irc_channel_move_near_server (server, channel_type,
-                                                  ptr_buffer);
+                    irc_buffer_move_near_server (
+                        server,
+                        0,  /* list_buffer */
+                        channel_type,
+                        ptr_buffer);
                     break;
             }
             if (ptr_buffer_for_merge)
@@ -475,17 +403,14 @@ irc_channel_create_buffer (struct t_irc_server *server,
         }
         if (channel_name_lower)
         {
-            weechat_hashtable_remove (server->join_manual, channel_name_lower);
             weechat_hashtable_remove (server->join_noswitch, channel_name_lower);
             free (channel_name_lower);
         }
     }
 
 end:
-    if (buffer_props)
-        weechat_hashtable_free (buffer_props);
-    if (buffer_name)
-        free (buffer_name);
+    weechat_hashtable_free (buffer_props);
+    free (buffer_name);
     return ptr_buffer;
 }
 
@@ -502,7 +427,8 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
 {
     struct t_irc_channel *new_channel;
     struct t_gui_buffer *ptr_buffer;
-    const char *ptr_chanmode;
+    const char *ptr_chanmode, *ptr_channel_key;
+    char *channel_name_lower;
 
     /* create buffer for channel (or use existing one) */
     ptr_buffer = irc_channel_create_buffer (server, channel_type,
@@ -526,15 +452,15 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
     new_channel->topic = NULL;
     new_channel->modes = NULL;
     new_channel->limit = 0;
-    if (weechat_hashtable_has_key (server->join_channel_key, channel_name))
+    new_channel->key = NULL;
+    channel_name_lower = weechat_string_tolower (channel_name);
+    if (channel_name_lower)
     {
-        new_channel->key = strdup (
-            weechat_hashtable_get (server->join_channel_key, channel_name));
-        weechat_hashtable_remove (server->join_channel_key, channel_name);
-    }
-    else
-    {
-        new_channel->key = NULL;
+        ptr_channel_key = weechat_hashtable_get (server->join_channel_key,
+                                                 channel_name_lower);
+        if (ptr_channel_key)
+            new_channel->key = strdup (ptr_channel_key);
+        free (channel_name_lower);
     }
     new_channel->join_msg_received = weechat_hashtable_new (
         32,
@@ -651,6 +577,121 @@ irc_channel_add_nicklist_groups (struct t_irc_server *server,
                                 "weechat.color.nicklist_group", 1);
 }
 
+/*
+ * Sets modes on channel buffer.
+ */
+
+void
+irc_channel_set_buffer_modes (struct t_irc_server *server,
+                              struct t_irc_channel *channel)
+{
+    char *modes_without_args;
+    const char *pos_space;
+
+    if (!server || !channel || !channel->buffer)
+        return;
+
+    if ((channel->type == IRC_CHANNEL_TYPE_CHANNEL)
+        && channel->nicks
+        && channel->modes && channel->modes[0]
+        && (strcmp (channel->modes, "+") != 0))
+    {
+        modes_without_args = NULL;
+        if (!irc_config_display_channel_modes_arguments (channel->modes))
+        {
+            pos_space = strchr (channel->modes, ' ');
+            if (pos_space)
+            {
+                modes_without_args = weechat_strndup (
+                    channel->modes, pos_space - channel->modes);
+            }
+        }
+        weechat_buffer_set (
+            channel->buffer,
+            "modes",
+            (modes_without_args) ? modes_without_args : channel->modes);
+        free (modes_without_args);
+    }
+    else
+    {
+        weechat_buffer_set (channel->buffer, "modes", "");
+    }
+}
+
+/*
+ * Gets input prompt for a channel (channel is optional: when channel is not
+ * given, the prefix for nick is not returned).
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+irc_channel_get_buffer_input_prompt (struct t_irc_server *server,
+                                     struct t_irc_channel *channel)
+{
+    struct t_irc_nick *ptr_nick;
+    int display_modes;
+    char str_prefix[64], *prompt;
+
+    if (!server || !server->nick)
+        return NULL;
+
+    /* build prefix */
+    str_prefix[0] = '\0';
+    if (channel
+        && (channel->type == IRC_CHANNEL_TYPE_CHANNEL)
+        && weechat_config_boolean (irc_config_look_item_nick_prefix))
+    {
+        ptr_nick = irc_nick_search (server, channel, server->nick);
+        if (ptr_nick)
+        {
+            if (weechat_config_boolean (irc_config_look_nick_mode_empty)
+                || (ptr_nick->prefix[0] != ' '))
+            {
+                snprintf (str_prefix, sizeof (str_prefix), "%s%s",
+                          weechat_color (
+                              irc_nick_get_prefix_color_name (
+                                  server, ptr_nick->prefix[0])),
+                          ptr_nick->prefix);
+            }
+        }
+    }
+
+    display_modes = (weechat_config_boolean (irc_config_look_item_nick_modes)
+                     && server->nick_modes && server->nick_modes[0]);
+
+    weechat_asprintf (&prompt, "%s%s%s%s%s%s%s%s%s",
+                      str_prefix,
+                      IRC_COLOR_INPUT_NICK,
+                      server->nick,
+                      (display_modes) ? IRC_COLOR_BAR_DELIM : "",
+                      (display_modes) ? "(" : "",
+                      (display_modes) ? IRC_COLOR_ITEM_NICK_MODES : "",
+                      (display_modes) ? server->nick_modes : "",
+                      (display_modes) ? IRC_COLOR_BAR_DELIM : "",
+                      (display_modes) ? ")" : "");
+
+    return prompt;
+}
+
+/*
+ * Sets input prompt on channel buffer.
+ */
+
+void
+irc_channel_set_buffer_input_prompt (struct t_irc_server *server,
+                                     struct t_irc_channel *channel)
+{
+    char *prompt;
+
+    if (!server || !channel || !channel->buffer)
+        return;
+
+    prompt = irc_channel_get_buffer_input_prompt (server, channel);
+    weechat_buffer_set (channel->buffer, "input_prompt", (prompt) ? prompt : "");
+    free (prompt);
+}
+
 /*
  * Sets the buffer title with the channel topic.
  */
@@ -666,8 +707,7 @@ irc_channel_set_buffer_title (struct t_irc_channel *channel)
             channel->topic,
             (weechat_config_boolean (irc_config_look_topic_strip_colors)) ? 0 : 1);
         weechat_buffer_set (channel->buffer, "title", title_color);
-        if (title_color)
-            free (title_color);
+        free (title_color);
     }
     else
     {
@@ -695,8 +735,7 @@ irc_channel_set_topic (struct t_irc_channel *channel, const char *topic)
         && topic && topic[0]
         && (strcmp (channel->topic, topic) != 0));
 
-    if (channel->topic)
-        free (channel->topic);
+    free (channel->topic);
     channel->topic = (topic) ? strdup (topic) : NULL;
 
     irc_channel_set_buffer_title (channel);
@@ -742,9 +781,7 @@ irc_channel_set_topic_private_buffers (struct t_irc_server *server,
 void
 irc_channel_set_modes (struct t_irc_channel *channel, const char *modes)
 {
-    if (channel->modes)
-        free (channel->modes);
-
+    free (channel->modes);
     channel->modes = (modes) ? strdup (modes) : NULL;
 }
 
@@ -828,8 +865,7 @@ irc_channel_remove_account (struct t_irc_server *server,
         for (ptr_nick = channel->nicks; ptr_nick;
              ptr_nick = ptr_nick->next_nick)
         {
-            if (ptr_nick->account)
-                free (ptr_nick->account);
+            free (ptr_nick->account);
             ptr_nick->account = NULL;
         }
     }
@@ -1074,8 +1110,7 @@ irc_channel_nick_speaking_time_free (struct t_irc_channel *channel,
         return;
 
     /* free data */
-    if (nick_speaking->nick)
-        free (nick_speaking->nick);
+    free (nick_speaking->nick);
 
     /* remove nick from list */
     if (nick_speaking->prev_nick)
@@ -1440,8 +1475,7 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
         line = weechat_hdata_move (hdata_line, line, -1);
     }
 
-    if (nick_to_search)
-        free (nick_to_search);
+    free (nick_to_search);
 
     weechat_hashtable_remove (channel->join_smart_filtered, nick);
 }
@@ -1535,10 +1569,18 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
                                      const char *nickname)
 {
     struct t_irc_channel *ptr_channel;
+    struct t_irc_protocol_ctxt ctxt;
 
     if (!server || (!nick && !nickname))
         return;
 
+    memset (&ctxt, 0, sizeof (ctxt));
+    ctxt.server = server;
+    ctxt.nick = (nick) ? nick->name : NULL;
+    ctxt.nick_is_me = (irc_server_strcasecmp (server, ctxt.nick, server->nick) == 0);
+    ctxt.address = (nick) ? nick->host : NULL;
+    ctxt.command = strdup ("nick_back");
+
     for (ptr_channel = server->channels; ptr_channel;
          ptr_channel = ptr_channel->next_channel)
     {
@@ -1551,13 +1593,7 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
                 weechat_printf_date_tags (
                     ptr_channel->buffer,
                     0,
-                    irc_protocol_tags (
-                        server,
-                        "nick_back",
-                        NULL,
-                        NULL,
-                        (nick) ? nick->name : NULL,
-                        (nick) ? nick->host : NULL),
+                    irc_protocol_tags (&ctxt, NULL),
                     _("%s%s%s %s(%s%s%s)%s is back on server"),
                     weechat_prefix ("join"),
                     irc_nick_color_for_msg (server, 1, nick, nickname),
@@ -1571,6 +1607,8 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
             ptr_channel->has_quit_server = 0;
         }
     }
+
+    free (ctxt.command);
 }
 
 /*
@@ -1620,31 +1658,19 @@ irc_channel_free (struct t_irc_server *server, struct t_irc_channel *channel)
     irc_modelist_free_all (channel);
 
     /* free channel data */
-    if (channel->name)
-        free (channel->name);
-    if (channel->topic)
-        free (channel->topic);
-    if (channel->modes)
-        free (channel->modes);
-    if (channel->key)
-        free (channel->key);
-    if (channel->join_msg_received)
-        weechat_hashtable_free (channel->join_msg_received);
-    if (channel->away_message)
-        free (channel->away_message);
-    if (channel->pv_remote_nick_color)
-        free (channel->pv_remote_nick_color);
-    if (channel->hook_autorejoin)
-        weechat_unhook (channel->hook_autorejoin);
-    if (channel->nicks_speaking[0])
-        weechat_list_free (channel->nicks_speaking[0]);
-    if (channel->nicks_speaking[1])
-        weechat_list_free (channel->nicks_speaking[1]);
+    free (channel->name);
+    free (channel->topic);
+    free (channel->modes);
+    free (channel->key);
+    weechat_hashtable_free (channel->join_msg_received);
+    free (channel->away_message);
+    free (channel->pv_remote_nick_color);
+    weechat_unhook (channel->hook_autorejoin);
+    weechat_list_free (channel->nicks_speaking[0]);
+    weechat_list_free (channel->nicks_speaking[1]);
     irc_channel_nick_speaking_time_free_all (channel);
-    if (channel->join_smart_filtered)
-        weechat_hashtable_free (channel->join_smart_filtered);
-    if (channel->buffer_as_string)
-        free (channel->buffer_as_string);
+    weechat_hashtable_free (channel->join_smart_filtered);
+    free (channel->buffer_as_string);
 
     free (channel);
 
@@ -1867,43 +1893,43 @@ irc_channel_print_log (struct t_irc_channel *channel)
     struct t_irc_modelist *ptr_modelist;
 
     weechat_log_printf ("");
-    weechat_log_printf ("  => channel %s (addr:0x%lx):", channel->name, channel);
-    weechat_log_printf ("       type . . . . . . . . . . : %d",    channel->type);
-    weechat_log_printf ("       topic. . . . . . . . . . : '%s'",  channel->topic);
-    weechat_log_printf ("       modes. . . . . . . . . . : '%s'",  channel->modes);
-    weechat_log_printf ("       limit. . . . . . . . . . : %d",    channel->limit);
-    weechat_log_printf ("       key. . . . . . . . . . . : '%s'",  channel->key);
-    weechat_log_printf ("       join_msg_received. . . . : 0x%lx (hashtable: '%s')",
+    weechat_log_printf ("  => channel %s (addr:%p):", channel->name, channel);
+    weechat_log_printf ("       type . . . . . . . . . . : %d", channel->type);
+    weechat_log_printf ("       topic. . . . . . . . . . : '%s'", channel->topic);
+    weechat_log_printf ("       modes. . . . . . . . . . : '%s'", channel->modes);
+    weechat_log_printf ("       limit. . . . . . . . . . : %d", channel->limit);
+    weechat_log_printf ("       key. . . . . . . . . . . : '%s'", channel->key);
+    weechat_log_printf ("       join_msg_received. . . . : %p (hashtable: '%s')",
                         channel->join_msg_received,
                         weechat_hashtable_get_string (channel->join_msg_received,
                                                       "keys_values"));
-    weechat_log_printf ("       checking_whox. . . . . . : %d",    channel->checking_whox);
-    weechat_log_printf ("       away_message . . . . . . : '%s'",  channel->away_message);
-    weechat_log_printf ("       has_quit_server. . . . . : %d",    channel->has_quit_server);
-    weechat_log_printf ("       cycle. . . . . . . . . . : %d",    channel->cycle);
-    weechat_log_printf ("       part . . . . . . . . . . : %d",    channel->part);
-    weechat_log_printf ("       nick_completion_reset. . : %d",    channel->nick_completion_reset);
-    weechat_log_printf ("       pv_remote_nick_color . . : '%s'",  channel->pv_remote_nick_color);
-    weechat_log_printf ("       hook_autorejoin. . . . . : 0x%lx", channel->hook_autorejoin);
-    weechat_log_printf ("       nicks_count. . . . . . . : %d",    channel->nicks_count);
-    weechat_log_printf ("       nicks. . . . . . . . . . : 0x%lx", channel->nicks);
-    weechat_log_printf ("       last_nick. . . . . . . . : 0x%lx", channel->last_nick);
-    weechat_log_printf ("       nicks_speaking[0]. . . . : 0x%lx", channel->nicks_speaking[0]);
-    weechat_log_printf ("       nicks_speaking[1]. . . . : 0x%lx", channel->nicks_speaking[1]);
-    weechat_log_printf ("       nicks_speaking_time. . . : 0x%lx", channel->nicks_speaking_time);
-    weechat_log_printf ("       last_nick_speaking_time. : 0x%lx", channel->last_nick_speaking_time);
-    weechat_log_printf ("       modelists. . . . . . . . : 0x%lx", channel->modelists);
-    weechat_log_printf ("       last_modelist. . . . . . : 0x%lx", channel->last_modelist);
-    weechat_log_printf ("       join_smart_filtered. . . : 0x%lx (hashtable: '%s')",
+    weechat_log_printf ("       checking_whox. . . . . . : %d", channel->checking_whox);
+    weechat_log_printf ("       away_message . . . . . . : '%s'", channel->away_message);
+    weechat_log_printf ("       has_quit_server. . . . . : %d", channel->has_quit_server);
+    weechat_log_printf ("       cycle. . . . . . . . . . : %d", channel->cycle);
+    weechat_log_printf ("       part . . . . . . . . . . : %d", channel->part);
+    weechat_log_printf ("       nick_completion_reset. . : %d", channel->nick_completion_reset);
+    weechat_log_printf ("       pv_remote_nick_color . . : '%s'", channel->pv_remote_nick_color);
+    weechat_log_printf ("       hook_autorejoin. . . . . : %p", channel->hook_autorejoin);
+    weechat_log_printf ("       nicks_count. . . . . . . : %d", channel->nicks_count);
+    weechat_log_printf ("       nicks. . . . . . . . . . : %p", channel->nicks);
+    weechat_log_printf ("       last_nick. . . . . . . . : %p", channel->last_nick);
+    weechat_log_printf ("       nicks_speaking[0]. . . . : %p", channel->nicks_speaking[0]);
+    weechat_log_printf ("       nicks_speaking[1]. . . . : %p", channel->nicks_speaking[1]);
+    weechat_log_printf ("       nicks_speaking_time. . . : %p", channel->nicks_speaking_time);
+    weechat_log_printf ("       last_nick_speaking_time. : %p", channel->last_nick_speaking_time);
+    weechat_log_printf ("       modelists. . . . . . . . : %p", channel->modelists);
+    weechat_log_printf ("       last_modelist. . . . . . : %p", channel->last_modelist);
+    weechat_log_printf ("       join_smart_filtered. . . : %p (hashtable: '%s')",
                         channel->join_smart_filtered,
                         weechat_hashtable_get_string (channel->join_smart_filtered,
                                                       "keys_values"));
-    weechat_log_printf ("       typing_state . . . . . . : %d",    channel->typing_state);
-    weechat_log_printf ("       typing_status_sent . . . : %lld",  (long long)channel->typing_status_sent);
-    weechat_log_printf ("       buffer . . . . . . . . . : 0x%lx", channel->buffer);
-    weechat_log_printf ("       buffer_as_string . . . . : '%s'",  channel->buffer_as_string);
-    weechat_log_printf ("       prev_channel . . . . . . : 0x%lx", channel->prev_channel);
-    weechat_log_printf ("       next_channel . . . . . . : 0x%lx", channel->next_channel);
+    weechat_log_printf ("       typing_state . . . . . . : %d", channel->typing_state);
+    weechat_log_printf ("       typing_status_sent . . . : %lld", (long long)channel->typing_status_sent);
+    weechat_log_printf ("       buffer . . . . . . . . . : %p", channel->buffer);
+    weechat_log_printf ("       buffer_as_string . . . . : '%s'", channel->buffer_as_string);
+    weechat_log_printf ("       prev_channel . . . . . . : %p", channel->prev_channel);
+    weechat_log_printf ("       next_channel . . . . . . : %p", channel->next_channel);
     for (i = 0; i < 2; i++)
     {
         if (channel->nicks_speaking[i])
